@@ -3,17 +3,17 @@
 use core::ops::*;
 use num_traits::{One, Zero};
 
-/// Alias of the builtin integer type with max width
+/// Alias of the builtin integer type with max width (currently [u128])
 #[allow(non_camel_case_types)]
 pub type umax = u128;
 
 #[allow(non_camel_case_types)]
 #[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord)]
-/// A double width integer type based on the largest built-in integer (u128).
-/// It's used to support double-width operations on u128.
+/// A double width integer type based on the largest built-in integer type [umax] (currently [u128]), and
+/// to support double-width operations on it is the only goal for this type.
 ///
 /// Although it can be regarded as u256, it's not as feature-rich as in other crates
-/// since it's only designed to support this crate.
+/// since it's only designed to support this crate and few other crates (will be noted in comments).
 pub struct udouble {
     /// Most significant part
     pub hi: umax,
@@ -21,6 +21,11 @@ pub struct udouble {
     pub lo: umax,
 }
 
+// TODO: port optimizations from and benchmark against
+//       https://github.com/nlordell/ethnum-rs
+// TODO(v0.3): remove excessive APIs that are not (likely) used (to reduce optimization burden)
+// if we found that we have complex requirements for mul and div, migrate to use ethnum
+
 impl udouble {
     pub fn widening_add(lhs: umax, rhs: umax) -> Self {
         let (sum, carry) = lhs.overflowing_add(rhs);
@@ -59,9 +64,16 @@ impl udouble {
         let (z1x, c1x) = u128::overflowing_mul(self.lo, rhs.hi);
         let (z1y, c1y) = u128::overflowing_mul(self.hi, rhs.lo);
         let (z1z, c1z) = u128::overflowing_add(z1x, z1y);
-        let (z1, cs1) = z1z.overflowing_add(c0);
-        let (z1, cs2) = z1.overflowing_add(c1z as umax);
-        (Self { hi: z1, lo: z0 }, c1x | c1y | cs1 | cs2 | c2)
+        let (z1, c1) = z1z.overflowing_add(c0);
+        (Self { hi: z1, lo: z0 }, c1x | c1y | c1z | c1 | c2)
+    }
+
+    /// Multiplication of double width and single width
+    pub fn overflowing_muls(&self, rhs: umax) -> (Self, bool) {
+        let Self { lo: z0, hi: c0 } = Self::widening_mul(self.lo, rhs);
+        let (z1, c1) = u128::overflowing_mul(self.hi, rhs);
+        let (z1, cs1) = z1.overflowing_add(c0);
+        (Self { hi: z1, lo: z0 }, c1 | c1 | cs1)
     }
 
     pub fn checked_shl(self, rhs: u32) -> Option<Self> {
@@ -159,73 +171,79 @@ impl One for udouble {
     }
 }
 
-impl Shl<u32> for udouble {
-    type Output = Self;
-    fn shl(self, rhs: u32) -> Self::Output {
-        match rhs {
-            0 => self,
-            s if s >= umax::BITS => Self {
-                hi: self.lo << (s - umax::BITS),
-                lo: 0,
-            },
-            s => Self {
-                lo: self.lo << s,
-                hi: (self.hi << s) | (self.lo >> (umax::BITS - s)),
-            },
+macro_rules! impl_shops {
+    ($t:ty) => {
+        impl Shl<$t> for udouble {
+            type Output = Self;
+            fn shl(self, rhs: $t) -> Self::Output {
+                match rhs {
+                    0 => self,
+                    s if s >= umax::BITS as $t => Self {
+                        hi: self.lo << (s - umax::BITS as $t),
+                        lo: 0,
+                    },
+                    s => Self {
+                        lo: self.lo << s,
+                        hi: (self.hi << s) | (self.lo >> (umax::BITS as $t - s)),
+                    },
+                }
+            }
         }
-    }
+        impl ShlAssign<$t> for udouble {
+            fn shl_assign(&mut self, rhs: $t) {
+                match rhs {
+                    0 => {}
+                    s if s >= umax::BITS as $t => {
+                        self.hi = self.lo << (s - umax::BITS as $t);
+                        self.lo = 0;
+                    }
+                    s => {
+                        self.hi <<= s;
+                        self.hi |= self.lo >> (umax::BITS as $t - s);
+                        self.lo <<= s;
+                    }
+                }
+            }
+        }
+        impl Shr<$t> for udouble {
+            type Output = Self;
+            fn shr(self, rhs: $t) -> Self::Output {
+                match rhs {
+                    0 => self,
+                    s if s >= umax::BITS as $t => Self {
+                        lo: self.hi >> (rhs - umax::BITS as $t),
+                        hi: 0,
+                    },
+                    s => Self {
+                        hi: self.hi >> s,
+                        lo: (self.lo >> s) | (self.hi << (umax::BITS as $t - s)),
+                    },
+                }
+            }
+        }
+        impl ShrAssign<$t> for udouble {
+            fn shr_assign(&mut self, rhs: $t) {
+                match rhs {
+                    0 => {}
+                    s if s >= umax::BITS as $t => {
+                        self.lo = self.hi >> (rhs - umax::BITS as $t);
+                        self.hi = 0;
+                    }
+                    s => {
+                        self.lo >>= s;
+                        self.lo |= self.hi << (umax::BITS as $t - s);
+                        self.hi >>= s;
+                    }
+                }
+            }
+        }
+    };
 }
 
-impl ShlAssign<u32> for udouble {
-    fn shl_assign(&mut self, rhs: u32) {
-        match rhs {
-            0 => {}
-            s if s >= umax::BITS => {
-                self.hi = self.lo << (s - umax::BITS);
-                self.lo = 0;
-            }
-            s => {
-                self.hi <<= s;
-                self.hi |= self.lo >> (umax::BITS - s);
-                self.lo <<= s;
-            }
-        }
-    }
-}
-
-impl Shr<u32> for udouble {
-    type Output = Self;
-    fn shr(self, rhs: u32) -> Self::Output {
-        match rhs {
-            0 => self,
-            s if s >= umax::BITS => Self {
-                lo: self.hi >> (rhs - umax::BITS),
-                hi: 0,
-            },
-            s => Self {
-                hi: self.hi >> s,
-                lo: (self.lo >> s) | (self.hi << (umax::BITS - s)),
-            },
-        }
-    }
-}
-
-impl ShrAssign<u32> for udouble {
-    fn shr_assign(&mut self, rhs: u32) {
-        match rhs {
-            0 => {}
-            s if s >= umax::BITS => {
-                self.lo = self.hi >> (rhs - umax::BITS);
-                self.hi = 0;
-            }
-            s => {
-                self.lo >>= s;
-                self.lo |= self.hi << (umax::BITS - s);
-                self.hi >>= s;
-            }
-        }
-    }
-}
+// only implement most useful ones, so that we don't need to optimize so many variants
+impl_shops!(u8);
+impl_shops!(u16);
+impl_shops!(u32);
 
 impl BitAnd for udouble {
     type Output = Self;
@@ -277,7 +295,7 @@ impl Not for udouble {
 }
 
 impl udouble {
-    pub fn leading_zeros(self) -> u32 {
+    pub const fn leading_zeros(self) -> u32 {
         if self.hi == 0 {
             self.lo.leading_zeros() + umax::BITS
         } else {
@@ -291,8 +309,8 @@ impl udouble {
         let mut d = other; // denominator
         let mut q = Self::zero(); // quotient
 
-        let nbits = 2 * umax::BITS - n.leading_zeros();
-        let dbits = 2 * umax::BITS - d.leading_zeros();
+        let nbits = (2 * umax::BITS - n.leading_zeros()) as u16; // assuming umax = u128
+        let dbits = (2 * umax::BITS - d.leading_zeros()) as u16;
         assert!(dbits != 0, "division by zero");
 
         // Early return in case we are dividing by a larger number than us
@@ -312,8 +330,8 @@ impl udouble {
                 break;
             }
 
-            d >>= 1;
-            q <<= 1;
+            d >>= 1u8;
+            q <<= 1u8;
             shift -= 1;
         }
         (q, n)
@@ -323,6 +341,7 @@ impl udouble {
 impl Mul for udouble {
     type Output = Self;
     fn mul(self, rhs: Self) -> Self::Output {
+        // TODO: use checked_mul instead of overflowing_mul
         let (m, overflow) = self.overflowing_mul(rhs);
         assert!(!overflow, "multiplication overflow!");
         m
@@ -343,6 +362,15 @@ impl Rem for udouble {
     }
 }
 
+impl Mul<umax> for udouble {
+    type Output = Self;
+    fn mul(self, rhs: umax) -> Self::Output {
+        let (m, overflow) = self.overflowing_muls(rhs);
+        assert!(!overflow, "multiplication overflow!");
+        m
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -352,6 +380,7 @@ mod tests {
     fn test_construction() {
         // construct zero
         assert!(udouble { lo: 0, hi: 0 }.is_zero());
+        assert!(udouble { lo: 1, hi: 0 }.is_one());
 
         // from widening operators
         assert_eq!(udouble { hi: 0, lo: 2 }, udouble::widening_add(1, 1));
@@ -380,9 +409,10 @@ mod tests {
         assert_eq!(TWOZERO - ONEMAX, ONE);
 
         assert_eq!(ONE << umax::BITS, ONEZERO);
-        assert_eq!((MAX << 1) + 1, ONEMAX);
+        assert_eq!((MAX << 1u8) + 1, ONEMAX);
+        assert_eq!(ONE << 200u8, udouble { lo: 0, hi: 1 << (200 - umax::BITS) });
         assert_eq!(ONEZERO >> umax::BITS, ONE);
-        assert_eq!(ONEMAX >> 1, MAX);
+        assert_eq!(ONEMAX >> 1u8, MAX);
 
         assert_eq!(ONE * MAX, MAX);
         assert_eq!(ONE * ONEMAX, ONEMAX);
@@ -402,6 +432,9 @@ mod tests {
         let (m, overflow) = ONEMAX.overflowing_mul(MAX);
         assert_eq!(m, udouble { lo: 1, hi: umax::MAX - 2});
         assert!(overflow);
+        let (m, overflow) = TWOZERO.overflowing_mul(MAX);
+        assert_eq!(m, udouble { lo: 0, hi: umax::MAX - 1});
+        assert!(overflow);
     }
 
     #[test]
