@@ -24,7 +24,7 @@ pub trait Montgomery: Sized {
     type Double;
 
     /// Calculate -(m^-1) mod R
-    fn neginv(m: &Self) -> Self::Inv;
+    fn neginv(m: &Self) -> Self::Inv; // TODO(v0.3): return Option<Self::Inv>, return None if m is even
 
     /// Transform a normal integer into Montgomery form (compute `target*R mod m`)
     fn transform(target: Self, m: &Self) -> Self;
@@ -251,8 +251,7 @@ impl Montgomery for u128 {
     impl_uprim_montgomery!();
 }
 
-/// An integer represented in Montgomery form, it implements [ModularInteger] interface
-/// and it's generally more efficient than the vanilla integer in modular operations.
+/// An integer represented in [Montgomery form](https://en.wikipedia.org/wiki/Montgomery_modular_multiplication#Montgomery_form).
 #[derive(Debug, Clone, Copy)]
 pub struct MontgomeryInt<T: Integer + Montgomery> {
     /// The Montgomery representation of the integer.
@@ -265,14 +264,6 @@ pub struct MontgomeryInt<T: Integer + Montgomery> {
     mi: T::Inv,
 }
 
-/// A word-size integer in Montgomery form with fixed modulus
-// TODO: implement after we have const implementation of invm
-#[derive(Debug, Clone, Copy)]
-struct MontgomeryWord<const M: usize>(usize);
-
-// XXX: we can also implement MontgomeryMersenne<const M: usize> to support Montgomery form
-// with (Pseudo) Mersenne prime as modulo. REF: https://eprint.iacr.org/2018/1038.pdf
-
 impl<T: Integer + Montgomery> MontgomeryInt<T> {
     #[inline]
     fn check_modulus_eq(&self, rhs: &Self) {
@@ -394,7 +385,7 @@ where
     }
 
     #[inline]
-    fn new(&self, n: T) -> Self {
+    fn new(&self, n: T) -> Self { // TODO(v0.3): rename to convert
         let a = Montgomery::transform(n, &self.m);
         MontgomeryInt {
             a,
@@ -403,3 +394,88 @@ where
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use crate::ModularCoreOps;
+    use rand::random;
+
+    #[test]
+    fn creation_test() {
+        // a deterministic test case for u128
+        let a = (0x81u128 << 120) - 1;
+        let m = (0x81u128 << 119) - 1;
+        let m = m >> m.trailing_zeros();
+        assert_eq!(MontgomeryInt::new(a, m).residue(), a % m);
+
+        // random creation test
+        for _ in 0..10 {
+            let a = random::<u8>();
+            let m = random::<u8>() | 1;
+            assert_eq!(MontgomeryInt::new(a, m).residue(), a % m);
+    
+            let a = random::<u16>();
+            let m = random::<u16>() | 1;
+            assert_eq!(MontgomeryInt::new(a, m).residue(), a % m);
+    
+            let a = random::<u32>();
+            let m = random::<u32>() | 1;
+            assert_eq!(MontgomeryInt::new(a, m).residue(), a % m);
+    
+            let a = random::<u64>();
+            let m = random::<u64>() | 1;
+            assert_eq!(MontgomeryInt::new(a, m).residue(), a % m);
+    
+            let a = random::<u128>();
+            let m = random::<u128>() | 1;
+            assert_eq!(MontgomeryInt::new(a, m).residue(), a % m);
+        }
+    }
+
+    #[test]
+    fn binary_op_tests() {
+        // TODO(v0.3): test more operations
+        for _ in 0..10 {
+            let m = random::<u8>() | 1;
+            let (a, b) = (random::<u8>(), random::<u8>());
+            let am = MontgomeryInt::new(a, m);
+            let bm = am.new(b);
+            assert_eq!((am + bm).residue(), a.addm(b, &m));
+            assert_eq!((am - bm).residue(), a.subm(b, &m));
+            assert_eq!((am * bm).residue(), a.mulm(b, &m));
+
+            let m = random::<u16>() | 1;
+            let (a, b) = (random::<u16>(), random::<u16>());
+            let am = MontgomeryInt::new(a, m);
+            let bm = am.new(b);
+            assert_eq!((am + bm).residue(), a.addm(b, &m));
+            assert_eq!((am - bm).residue(), a.subm(b, &m));
+            assert_eq!((am * bm).residue(), a.mulm(b, &m));
+    
+            let m = random::<u32>() | 1;
+            let (a, b) = (random::<u32>(), random::<u32>());
+            let am = MontgomeryInt::new(a, m);
+            let bm = am.new(b);
+            assert_eq!((am + bm).residue(), a.addm(b, &m));
+            assert_eq!((am - bm).residue(), a.subm(b, &m));
+            assert_eq!((am * bm).residue(), a.mulm(b, &m));
+
+            let m = random::<u64>() | 1;
+            let (a, b) = (random::<u64>(), random::<u64>());
+            let am = MontgomeryInt::new(a, m);
+            let bm = am.new(b);
+            assert_eq!((am + bm).residue(), a.addm(b, &m));
+            assert_eq!((am - bm).residue(), a.subm(b, &m));
+            assert_eq!((am * bm).residue(), a.mulm(b, &m));
+    
+            let m = random::<u128>() | 1;
+            let (a, b) = (random::<u128>(), random::<u128>());
+            let am = MontgomeryInt::new(a, m);
+            let bm = am.new(b);
+            assert_eq!((am + bm).residue(), a.addm(b, &m));
+            assert_eq!((am - bm).residue(), a.subm(b, &m));
+            assert_eq!((am * bm).residue(), a.mulm(b, &m));
+        }
+    }
+}
